/*	******************************************************************
	SCOPE OF A VARIABLE 
	in Procedural Programming: Local vs Global
	******************************************************************
	-	This program demonstrates the SCOPE of Variables in 
		Procedural Programming
		-	i.e. The region in which they are accesible
	-	In Procedural Programming, we have
		-	Local Variable
			-	Accessible only in the Function where they are 
				defined
		-	Global Variable
			-	Accessible Everywhere
			-	AVOID whenever possible
				-	Anything in the entire program can access them
					any time
				-	Debugging can become a near impossibility!
*/
#include <iostream>
using namespace std;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Function DECLARATIONS
//	-	Signatures are different
//
int calc_Tax_Tickets( int, int );
double calc_Tax_Service( double, int );

//
//	Global Variable - Avoid wherever possible
//
double tax_Rate = 0.3;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
int main()
{
	//
	// Function Invocations

	//================================================================
	double charge_IT_perHour = 250;
	int num_Hours_IT = 12;

	//	Compiler knows which one to call since there is ONE function 
	//	-	called calc_Tax which 
	//		-	takes in a double and an int in that order
	//		-	returns a double
	//
	double tax_IT = calc_Tax_Service( charge_IT_perHour, num_Hours_IT );

	cout << "IT Services: The tax payable, for " 
		 << num_Hours_IT << " hours, at a charge of "
		 << charge_IT_perHour << " per hour will be "
		 << tax_IT
		 << endl;
	//================================================================
	int cost_Ticket_rugbyMatch = 55;
	int num_People_rugbyMatch = 22;

	//	Compiler knows which one to call since there is ONE function 
	//	-	called calc_Tax which 
	//		-	takes in two ints
	//		-	returns an int
	//
	int tax_rugbyMatch = calc_Tax_Tickets(	cost_Ticket_rugbyMatch,
											num_People_rugbyMatch
										 );

	cout << "Rugby Match: The tax payable, to take " 
		 << num_People_rugbyMatch << " people, each ticket costing "
		 << cost_Ticket_rugbyMatch << " will be "
		 << tax_rugbyMatch
		 << endl;

	//
	cout << endl << endl << "===========" 
		 << endl 
		 << "END...C++ Demo: Functions, No Inputs, No Output" 
		 << " " 
		 << endl;
	while(true){} 
	return 0;

}//END...Function MAIN


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Function DEFINITIONS
//
int calc_Tax_Tickets( int ticketPrice, int numPeople )
{
	int total_Price = ticketPrice * numPeople;
	int tax_Amount = tax_Rate * total_Price;

	tax_Rate = 0.35;
			// tax_Rate is a GLOBAL variable - can be changed here

	// total_Income = 750.0;	
			// ILLEGAL - total_Income is LOCAL to the function 
			// calc_Tax_Service - cannot be accessed here

	return ( tax_Amount );

}//END Function int calc_Tax( int, int )

double calc_Tax_Service( double chargePerHour, int num_Hours )
{
	double total_Income = chargePerHour * 
							static_cast<double>(num_Hours);
	double tax_Amount = tax_Rate * total_Income;

	tax_Rate = 0.35;
			// tax_Rate is a GLOBAL variable - can be changed here

	// total_Price = 750.0;	
			// ILLEGAL - total_Price is LOCAL to the function 
			// calc_Tax_Tickets - cannot be accessed here

	return ( tax_Amount );

}//END Function double calc_Tax( double, int )


//**************************************************************************
//	This software code is from the book Cybernetics in C++
//	by Avi Bhattacharyya,
//	Published by River Publishers
//	All Rights Reserved.
//
//	(C) Copyright River Publishers (2018 onwards)
//
//	-	Permission to use, copy, distribute and sell this software,
//		in whole or in part, is granted on condition this Copyright
//		Statement is included in all copies where any part is
//		present
//	-	The software is provided AS-IS - no recommendations are
//		made for its suitability for any purpose
//	-	No Warranties are given explicitly or implied
//	-	No responsibility is accepted for results obtained following
//		its usage in any way whatsoever
//**************************************************************************
