/*	******************************************************************
	POINTERS AS FUNCTION INPUTS
	******************************************************************
	-	Demonstrates how to implement a Function
	-	Shows the famous "swap scenario"
		-	The Addresses, not the Values, need to be passed
		-	They are passed as POINTERS here
			-	Can also be passed as References in C++
				-	Example of this in the References section
*/
#include <iostream>
using namespace std;

void swap_usingPointers( int *, int * );

int main()
{
	//
	int myInt_A = 7, myInt_B = 9;

	cout << "Before the attempted swap myInt_A = " << myInt_A  
		 << " and myInt_B = " << myInt_B << endl;

	//
	/*	INVOKING swap_usingPointers - inputs Pointers
		-	Want the inputs to be 
			-	"The ADDRESS of myInt_A", 
			-	"The ADDRESS of myInt_B"
		-	We say &myInt_A and &myInt_B, since 
			-	the operator to say "The address of a variable 
				pointed to by a pointer": AMPERSAND &
	*/
	swap_usingPointers( &myInt_A, &myInt_B ); 

	//
	cout << "After the attempted swap myInt_A = " << myInt_A  
		 << " and myInt_B = " << myInt_B << endl;

	//
	cout << endl << "===========" 
		 << endl << "END...C++ Pointers as Function Inputs Demo" 
		 << endl;
	while(true){}
	return 0;

}//END...function main


/*	==================================================================
	Function DEFINITION swap_usingPointers
	-	We are now going to pass the variables themselves, 
		not their values
	-	When a variable is defined, (for the best part) it stays in 
		the SAME memory location until it goes out of scope
	-	Thus passing this memory ADDRESS to the function is the way 
		we ensure the variable itself is passed
		-	Here we achieve this using Pointers
*/ 
void swap_usingPointers( int *ptrInt_In_A, int *ptrInt_In_B )
{
	int temp = *ptrInt_In_B;
	*ptrInt_In_B = *ptrInt_In_A;
	*ptrInt_In_A = temp;

}// END...function swap_usingPointers

//**************************************************************************
//	This software code is from the book Cybernetics in C++
//	by Avi Bhattacharyya,
//	Published by River Publishers
//	All Rights Reserved.
//
//	(C) Copyright River Publishers (2018 onwards)
//
//	-	Permission to use, copy, distribute and sell this software,
//		in whole or in part, is granted on condition this Copyright
//		Statement is included in all copies where any part is
//		present
//	-	The software is provided AS-IS - no recommendations are
//		made for its suitability for any purpose
//	-	No Warranties are given explicitly or implied
//	-	No responsibility is accepted for results obtained following
//		its usage in any way whatsoever
//**************************************************************************
 
