/*	******************************************************************
	HEAP SIMPLE USAGE
	******************************************************************
	-	This program places data on the Runtime Heap
	-	Performs calculations on this data
		-	using Pointers which are themselves on the Stack
		-	Stores the result on the Runtime Stack
	-	Frees up the Heap Memory
	-	As a safety measure, ensures the used Pointers are set to
		NULL
*/
#include <iostream>
#include <string>
using namespace std;

int main( )
{
	//
	//	Placing data on the Heap
	//
	int *ptr_numPeople = new int(5);
	double *ptr_totalCost = new double(85.5);

	cout << "The number of people, in the Runtime Heap, pointed to by" 
		 << " ptr_numPeople is " << *ptr_numPeople 
		 << endl;
	cout << "The total cost, in the Runtime Heap, pointed to by " 
		 << "ptr_totalCost is " << *ptr_totalCost 
		 << endl;

	//	Using the data on the Heap
	//	-	The result is being stored on the Runtime Stack
	double pricePerPerson = *ptr_totalCost / 
								static_cast<double>(*ptr_numPeople);
	cout << "The price per person is " << pricePerPerson << endl;

	//
	//	Freeing up the Heap - do not forget this! :-)
	//
	delete ptr_numPeople;
	delete ptr_totalCost;

	// Good Practice: Set the now Dangling Pointers to NULL
	ptr_numPeople = NULL;
	ptr_totalCost = NULL;

	//
	cout << endl << endl << "===========" 
		 << endl 
		 << "END...C++ Demo: Heap Simple Usage" 
		 << " " 
		 << endl;
	while(true){}
	return 0;

}//END...Function MAIN

//**************************************************************************
//	This software code is from the book Cybernetics in C++
//	by Avi Bhattacharyya,
//	Published by River Publishers
//	All Rights Reserved.
//
//	(C) Copyright River Publishers (2018 onwards)
//
//	-	Permission to use, copy, distribute and sell this software,
//		in whole or in part, is granted on condition this Copyright
//		Statement is included in all copies where any part is
//		present
//	-	The software is provided AS-IS - no recommendations are
//		made for its suitability for any purpose
//	-	No Warranties are given explicitly or implied
//	-	No responsibility is accepted for results obtained following
//		its usage in any way whatsoever
//**************************************************************************
 
