 
#include "Employee.h"

// ---------- STATIC Data Member(s) ------------
int Employee::num_Employees = 0;


// ---------- CONSTRUCTOR(S) ------------
Employee::Employee( string name_IN )
{
	name = name_IN;

	// Keeping track of the number of objects
	++num_Employees;

}//END...Constructor Employee( string name_IN )


Employee::Employee( string name_IN, string nI_Number_IN )
{
	name = name_IN;
	nI_Number = nI_Number_IN;

	// Keeping track of the number of objects
	++num_Employees;

}//END...Constructor Employee( string name_IN )

Employee::Employee( string name_IN, 
					string nI_Number_IN, 
					double salary_Gross_IN )
{
	name = name_IN;
	nI_Number = nI_Number_IN;
	salary_Gross = salary_Gross_IN;

	//-----------------------------------------------------
	//	Using the Inner Class TaxCalculations
	//	-	Object required, initialised with this pointer
	//		-	This holds the data of the current object
	//
	TaxCalculations tax_Results( this );

	tax_Results.calc_tax_Rate();
	tax_Results.calc_tax_Amount();
	tax_Results.calc_salary_Net();


	// Keeping track of the number of objects
	++num_Employees;

}/* END...Constructor Employee( string name_IN, 
								string nI_Num_IN, 
								double salary_Gross_IN )
 */

// ------- COPY Constructor ---------
Employee::Employee( Employee& ref_Employee_IN )
{
	name = ref_Employee_IN.name;
	nI_Number = ref_Employee_IN.nI_Number;
	salary_Gross = ref_Employee_IN.salary_Gross;
	tax_Rate = ref_Employee_IN.tax_Rate;
	tax_Amount = ref_Employee_IN.tax_Amount;
	salary_Net = ref_Employee_IN.salary_Net;

	cout << "Copy Constructor called for " << name << endl;
	
}// END...COPY Constructor

// ---------- DESTRUCTOR ------------
Employee::~Employee(  )
{
	cout << "Goodbye, I, " << name 
		 << " am  going out of scope now." 
		 << endl;

	// Keeping track of the number of objects
	--num_Employees;

}// END...Destructor ~Employee

// ---------- MUTATORS ------------
// const keyword is required here as well
double Employee::get_salary_Gross(  ) const
{
	// tax_Rate = 0.6;
		// ILLEGAL - const member function cannot
		//			 modify any data members

	return salary_Gross;

}//END...Mutator get_salary_Gross


void Employee::set_salary_Gross( double salary_Gross_IN )
{
	salary_Gross = salary_Gross_IN;

	//-----------------------------------------------------
	//	Calculating the NEW Tax Rate...
	//	...Using the Inner Class TaxCalculations
	//	-	Object required, initialised with this pointer
	//		-	This holds the data of the current object
	//
	TaxCalculations tax_Results( this );

	tax_Results.calc_tax_Rate();
	tax_Results.calc_tax_Amount();
	tax_Results.calc_salary_Net();

}//END...Mutator set_salary_Gross


// ---------- STATIC MEMBER FUNCTION(S) ------------
// ...also a Mutator
int Employee::get_num_Employees()
{
	return num_Employees;

}//END...Mutator get_num_Employees


// ---------- "NORMAL Member Functions " ------------
void Employee::greet()	
{
	cout << "Hello, I'm an Employee" << endl;

}//END...Member Function greet

void Employee::greet_Name()	
{
	cout << "Hello, I'm an Employee, " 
		 << "my name is " << name 
		 << endl;

}//END...Member Function greet_Name

void Employee::greet_Name_NINumber()
{
	cout << "Hello, I'm an Employee, " 
		 << "my name is " << name 
		 << " and my NI Number is " << nI_Number
		 << endl;

}//END...Member Function greet_Name_NINumber


void Employee::greet_Name_SalaryGross()
{
	cout << "Hello, I'm an Employee, my name is " << name
		 << ", my Gross Salary is " << salary_Gross
		 << endl;

}//END...Member Function greet_Name_SalaryGross()


void Employee::greet_AllAttributes()
{
	cout << endl
		 << "=====================" 
		 << endl
		 << "Hello, I am an employee, my name is " << name << ", " 
		 << endl 
		 << "my NI Number is " << nI_Number << "," 
		 << endl
		 << "my Gross Salary is " << salary_Gross 
		 << endl
		 << "my tax Rate is " << tax_Rate << "%" 
		 << endl
		 << "so the amount I have to pay in tax is " << tax_Amount 
		 << endl
		 << "thus my Net Salary is " << salary_Net 
		 << endl
		 << "=====================" << endl;
}// END...MF greet_AllAttributes()


void Employee::display_salary_Net()
{
	cout << "Accessing the Data Member salary_Net of :" 
		 << name << endl;

	cout << "-\tImplicitly\t		salary_Net = " 
		 << salary_Net << endl;

	cout << "-\tExplicitly\t		this -> salary_Net = " 
		 << this -> salary_Net << endl;

	cout << "-\tExplicitly\t		(*this).salary_Net = " 
		 << (*this).salary_Net << endl;

}// END...MF display_salary_Net()


/*================================================================================
	INNER CLASS: TaxCalculations
				void calc_tax_Rate();
				void calc_tax_Amount();
				void calc_salary_Net();
*/
Employee::TaxCalculations::TaxCalculations( Employee *ptr_Employee_IN )
{
	ptr_Employee = ptr_Employee_IN;

}//END...Constructor, Inner Class TaxCalculations


void Employee::TaxCalculations::calc_tax_Rate()
{
	// ptr_Employee -> salary_Gross;

	if( ptr_Employee-> salary_Gross < TAXRATE_THRESHOLD_LOWER )
	{
		ptr_Employee -> tax_Rate = TAXRATE_RATE_LOWER;
	}
	else
	{
		if( ptr_Employee-> salary_Gross < TAXRATE_THRESHOLD_INTERMEDIATE )
		{
			ptr_Employee -> tax_Rate = TAXRATE_RATE_INTERMEDIATE;
		}
		else
		{
			ptr_Employee -> tax_Rate = TAXRATE_RATE_HIGHER;
		}
	}
}//END...MF calc_tax_Rate

void Employee::TaxCalculations::calc_tax_Amount()
{
	ptr_Employee -> tax_Amount =	ptr_Employee -> salary_Gross * 
									ptr_Employee -> tax_Rate;

}//END...MF calc_tax_Amount


void Employee::TaxCalculations::calc_salary_Net()
{
	ptr_Employee -> salary_Net =	ptr_Employee -> salary_Gross - 
									ptr_Employee -> tax_Amount;

}//END...MF calc_tax_Amount

//END...INNER CLASS TaxCalculations

//**************************************************************************
//	This software code is from the book Cybernetics in C++
//	by Avi Bhattacharyya,
//	Published by River Publishers
//	All Rights Reserved.
//
//	(C) Copyright River Publishers (2018 onwards)
//
//	-	Permission to use, copy, distribute and sell this software,
//		in whole or in part, is granted on condition this Copyright
//		Statement is included in all copies where any part is
//		present
//	-	The software is provided AS-IS - no recommendations are
//		made for its suitability for any purpose
//	-	No Warranties are given explicitly or implied
//	-	No responsibility is accepted for results obtained following
//		its usage in any way whatsoever
//**************************************************************************

