/*	******************************************************************
	OOP Polymorphism on the Runtime Stack
	******************************************************************
	-	This program demonstrates using Objects based on Polymorphic 
		Classes
		-	Calling Member Functions via OBJECTS
			-	Polymorphism does not take effect here
		-	Calling Member Functions via POINTERS
			-	The Pointers are of type Pointer-to-BASEclass
			-	Polymorphism takes effect here
			-	The differences between virtual and non-virtual
				functions becomes apparent
*/

#include "Cat.h"
#include "Tiger.h"

#include <iostream>
using namespace std;


int main()
{
	//================================================================
	//	Defining Objects and Pointers to Objects
	//
	Cat oMalley;
	Cat *ptr_oMalley = &oMalley;
	
	Tiger shereKhan;
	Cat *ptr_shereKhan = &shereKhan;	//...

	//	...FINE - CAN have a Pointer-to-Cat containing the address of a Tiger
	//	-	Type FLEXIBILITY: Works because a Tiger IS-A Cat

	//================================================================
	//	(1)	Calling Member Functions via OBJECTS
	//
	cout << endl << "=============="
		 << endl << "Calling Member Functions via Objects"
		 << endl << "-------"
		 << endl;
	cout << "oMalley.greet()\t\t\t";
	oMalley.greet(); 			

	cout << "oMalley.sound()\t\t\t";
	oMalley.sound();			

	cout << "shereKhan.greet():\t\t";
	shereKhan.greet();			

	cout << "shereKhan.sound():\t\t";
	shereKhan.sound();			

	//================================================================
	//	(2)	Calling Member Functions via POINTERS
	//
	cout << endl << "=============="
		 << endl << "Calling Member Functions via POINTERS"
		 << endl << "-------"
		 << endl;

	cout << "ptr_oMalley -> greet()\t\t";
	ptr_oMalley -> greet();		

	cout << "ptr_oMalley -> sound()\t\t";
	ptr_oMalley -> sound();		
															
	cout << "ptr_shereKhan -> greet():\t";
	ptr_shereKhan -> greet();	

	cout << "ptr_shereKhan -> sound():\t";
	ptr_shereKhan -> sound();	

	//================================================================
	cout << endl << endl << "===========" << endl 
		 << "END...OOP Polymorphism, Runtime Stack Demo" 
		 << endl;
	while(true){}
	return 0;

}//END...Function main

//**************************************************************************
//	This software code is from the book Cybernetics in C++
//	by Avi Bhattacharyya,
//	Published by River Publishers
//	All Rights Reserved.
//
//	(C) Copyright River Publishers (2018 onwards)
//
//	-	Permission to use, copy, distribute and sell this software,
//		in whole or in part, is granted on condition this Copyright
//		Statement is included in all copies where any part is
//		present
//	-	The software is provided AS-IS - no recommendations are
//		made for its suitability for any purpose
//	-	No Warranties are given explicitly or implied
//	-	No responsibility is accepted for results obtained following
//		its usage in any way whatsoever
//**************************************************************************
 
