/*	******************************************************************
	OOP Polymorphism 
	with Pure Virtual Member Functions and Abstract Classes
	******************************************************************
	-	This program demonstrates
		-	Creating an Abstract Class with a Pure Virtual Member 
			Function
		-	Building concrete classes in an Inheritance hierarchy
		-	Implementing Polymorphism on the Runtime Stack
			-	Calling Member Functions via OBJECTS
			-	Calling Member Functions via POINTERS
		-	Implementing Polymorphism on the Runtime Heap
			-	Calling Member Functions via POINTERS
*/

#include "Cat.h"
#include "Tiger.h"
#include "Animal.h"

#include <iostream>
using namespace std;


int main()
{
	//================================================================
	//	(1)	POLYMORPHISM on the Runtime STACK
	//
	cout << "===================================" << endl 
		 << "POLYMORPHISM on the Runtime STACK" << endl
		 << "===================================" << endl;

	//================================================================
	//	Defining Objects and Pointers to Objects
	//

	// Animal fred;	//	ILLEGAL - Animal is an Abstract class, 
					//	cannot have an object of that type
	Cat oMalley;
	Cat *ptr_oMalley = &oMalley;
	
	Tiger shereKhan;
	Cat *ptr_shereKhan = &shereKhan;


	//================================================================
	//	(1.1)	Calling Member Functions via OBJECTS
	//
	cout << endl << "=============="
		 << endl << "Calling Member Functions via Objects"
		 << endl << "-------"
		 << endl;
	cout << "oMalley.greet()\t\t\t";
	oMalley.greet();			

	cout << "oMalley.sound()\t\t\t";
	oMalley.sound();			

	cout << "shereKhan.greet():\t\t";
	shereKhan.greet();			

	cout << "shereKhan.sound():\t\t";
	shereKhan.sound();			

	//================================================================
	//	(1.2)	Calling Member Functions via POINTERS
	//
	cout << endl << "=============="
		 << endl << "Calling Member Functions via POINTERS"
		 << endl << "-------"
		 << endl;

	cout << "ptr_oMalley -> greet()\t\t";
	ptr_oMalley -> greet();		

	cout << "ptr_oMalley -> sound()\t\t";
	ptr_oMalley -> sound();		
															
	cout << "ptr_shereKhan -> greet():\t";
	ptr_shereKhan -> greet();	

	cout << "ptr_shereKhan -> sound():\t";
	ptr_shereKhan -> sound();	

	//================================================================
	//	(2)	POLYMORPHISM on the Runtime HEAP
	//
	cout << endl
		 << "===================================" << endl 
		 << "POLYMORPHISM on the Runtime HEAP" << endl
		 << "===================================" << endl;

	//================================================================
	//	Defining Objects and Pointers to Objects
	//
	Cat *ptr_Duchess = new Cat();	// Cat twice 
	Cat *ptr_Rajah = new Tiger();

	//================================================================
	//	(2.1)	Call Member Functions via Pointers
	//
	cout << endl << "=============="
		 << endl << "Calling Member Functions via POINTERS"
		 << endl << "-------"
		 << endl;


	cout << "ptr_Duchess -> greet()\t\t";
	ptr_Duchess -> greet();		//	"Hello from the Cat World"
	
	cout << "ptr_Duchess -> sound()\t\t";
	ptr_Duchess -> sound();		//	"Miaow"
															
	
	cout << "ptr_Rajah -> greet()\t\t";
	ptr_Rajah -> greet();		//	"Hello from the Cat World"	
	
	cout << "ptr_Rajah -> sound()\t\t";
	ptr_Rajah -> sound();		//	"Tiger Roar RRRR"	



	//================================================================
	cout << endl << "===================" 
		 << endl << "END...OOP Polymorphism Demo" 
		 << endl << "...with Pure Virtual Member Functions "
				 << "and Abstract Classes";
	while(true){}
	return 0;

}//END...Function main

//**************************************************************************
//	This software code is from the book Cybernetics in C++
//	by Avi Bhattacharyya,
//	Published by River Publishers
//	All Rights Reserved.
//
//	(C) Copyright River Publishers (2018 onwards)
//
//	-	Permission to use, copy, distribute and sell this software,
//		in whole or in part, is granted on condition this Copyright
//		Statement is included in all copies where any part is
//		present
//	-	The software is provided AS-IS - no recommendations are
//		made for its suitability for any purpose
//	-	No Warranties are given explicitly or implied
//	-	No responsibility is accepted for results obtained following
//		its usage in any way whatsoever
//**************************************************************************
 
