/*	******************************************************************
	OOP Polymorphism in Data Structures
	******************************************************************
	-	This program demonstrates
		-	Creating a Data Structure (an array) of type
			Pointer-to-AbstractBaseClass
		-	Populating this with pointers to objects of different
			types in the OOP Inheritance Hierarchy
			-	These objects reside on both the
				-	Runtime Stack
				-	Runtime Heap
		-	Traversing this array and calling the same virtual
			member function for each element
		-	Components in this program:
			-	Pure Virtual Member Functions
			-	Abstract Classes

	-	NB. The Array is not the optimal Data Structure for Objects
		-	Better: STL Container, vector or deque
*/
#include <iostream>
using namespace std;

#include "Cat.h"
#include "Tiger.h"
#include "Animal.h"

int main()
{
	//================================================================
	//	(1)	POLYMORPHISM on the Runtime STACK
	//
	cout << "===================================" << endl 
		 << "POLYMORPHISM on the Runtime STACK" << endl
		 << "===================================" << endl;

	//================================================================
	//	Defining Objects and Pointers to Objects
	//

	// Animal fred;	//	ILLEGAL - Animal is an Abstract class, 
					//	cannot have an object of that type
	Cat oMalley;
	Cat *ptr_oMalley = &oMalley;
	
	Tiger shereKhan;
	Cat *ptr_shereKhan = &shereKhan;
	
	//================================================================
	//	(1.1)	Calling Member Functions via OBJECTS
	//
	cout << endl << "=============="
		 << endl << "Calling Member Functions via Objects"
		 << endl << "-------"
		 << endl;
	cout << "oMalley.greet()\t\t\t";
	oMalley.greet();			

	cout << "oMalley.sound()\t\t\t";
	oMalley.sound();			

	cout << "shereKhan.greet():\t\t";
	shereKhan.greet();			

	cout << "shereKhan.sound():\t\t";
	shereKhan.sound();			

	//================================================================
	//	(1.2)	Calling Member Functions via POINTERS
	//
	cout << endl << "=============="
		 << endl << "Calling Member Functions via POINTERS"
		 << endl << "-------"
		 << endl;

	cout << "ptr_oMalley -> greet()\t\t";
	ptr_oMalley -> greet();		

	cout << "ptr_oMalley -> sound()\t\t";
	ptr_oMalley -> sound();		
															
	cout << "ptr_shereKhan -> greet():\t";
	ptr_shereKhan -> greet();	

	cout << "ptr_shereKhan -> sound():\t";
	ptr_shereKhan -> sound();	

	//================================================================
	//	(2)	POLYMORPHISM on the Runtime HEAP
	//
	cout << endl
		 << "===================================" << endl 
		 << "POLYMORPHISM on the Runtime HEAP" << endl
		 << "===================================" << endl;

	//================================================================
	//	Defining Objects and Pointers to Objects
	//
	Cat *ptr_Duchess = new Cat();	// Cat twice 
	Cat *ptr_Rajah = new Tiger();

	//================================================================
	//	(2.1)	Call Member Functions via Pointers
	//
	cout << endl << "=============="
		 << endl << "Calling Member Functions via POINTERS"
		 << endl << "-------"
		 << endl;


	cout << "ptr_Duchess -> greet()\t\t";
	ptr_Duchess -> greet();		//	"Hello from the Cat World"
	
	cout << "ptr_Duchess -> sound()\t\t";
	ptr_Duchess -> sound();		//	"Miaow"
															
	
	cout << "ptr_Rajah -> greet()\t\t";
	ptr_Rajah -> greet();		//	"Hello from the Cat World"	
	
	cout << "ptr_Rajah -> sound()\t\t";
	ptr_Rajah -> sound();		//	"Tiger Roar RRRR"	


	//================================================================
	//	(3)	DATA STRUCTURE of Objects - the reason why  
	//		Polymorphism is essential
	//
	cout << endl
		 << "===========================================" << endl 
		 << "CREATING AN ARRAY of (Pointers-to-) Animals" << endl
		 << "===========================================" << endl;

	// Creating and initialising an array
	//	-	Name:	zoo
	//	-	Type:	Pointer-to-Animal
	//
	Animal *zoo[] = { ptr_oMalley, 
					  ptr_shereKhan, 
					  ptr_Duchess, 
					  ptr_Rajah 
					};
	
	cout << endl << "============="
		 << endl << "Calling each animal's sound() in the array zoo" 
		 << endl;

	// Not the best way
	//	- Better to use STL Iterators (C++ Part 2)
	for( int i = 0; i < (sizeof(zoo) / sizeof(Animal *)); ++i )	
	{
		zoo[i] -> sound();
	}

	//================================================================
	cout << endl << "===================" 
		 << endl << "END...OOP Polymorphism Demo" 
		 << endl << "...with Data Structures";
	while(true){}
	return 0;

}//END...Function main

//**************************************************************************
//	This software code is from the book Cybernetics in C++
//	by Avi Bhattacharyya,
//	Published by River Publishers
//	All Rights Reserved.
//
//	(C) Copyright River Publishers (2018 onwards)
//
//	-	Permission to use, copy, distribute and sell this software,
//		in whole or in part, is granted on condition this Copyright
//		Statement is included in all copies where any part is
//		present
//	-	The software is provided AS-IS - no recommendations are
//		made for its suitability for any purpose
//	-	No Warranties are given explicitly or implied
//	-	No responsibility is accepted for results obtained following
//		its usage in any way whatsoever
//**************************************************************************
 
